define([
    'backbone',
    'text!../templates/form-rules/form-rules-form.html',
    'text!../templates/form-rules/form-rules-field-row.html',
    '../collections/rule-forms',
    'moment',
    'i18n!../../../nls/dashboard_'+$('#main-dashboard-system-language').val()+'_ln'
], function(Backbone, FormRulesFormViewTmpl, FormRulesFieldRowTmpl, FormsCollection, moment, i18n){
    var LeadFormRulesView = Backbone.View.extend({
        el: $('#lead-form-create-form-rule-block'),
        events: {
            'change #forms-for-rules' : 'changeForm',
            'click #add-field-to-form-rule': 'addField',
            'click .form-field-rules-row-delete': 'deleteFieldRow',
            'click #form-actions-list-block li a': 'changeFormAction',
            'click #create-lead-form-rule': 'createLeadFormRule',
            'change .form-field-operator-value' : 'changeRuleOperator',
            'change #form-action-assign-task-immediate': 'switchImmediate',
            'change #form-action-assign-task-number-of-before-action-units': 'switchTaskUnits',
            'click .checkbox_radio-wrap' : 'switcherAction'
        },
        templates: {

        },
        initialize: function(){
            this.$el.unbind();
            this.$el = $('#lead-form-create-form-rule-block');
            this.$el.attr('action', $('#website_url').val()+'api/leads/leads');

            this.forms = new FormsCollection();
            this.forms.isGrid = true;
            this.forms.on('reset', this.renderConfigForm, this);
            this.forms.on('add', this.renderConfigForm, this);
            this.forms.on('destroy', this.renderConfigForm, this);
        },
        render: function(){
            this.forms.pager();
        },
        renderConfigForm: function()
        {
            this.forms.forms.leadOwnersData = this.alphabeticalSort(this.forms.forms.leadOwners);
            this.forms.forms.leadTagsData = this.alphabeticalSort(this.forms.forms.leadTags);
            this.forms.forms.lifecycleStagesData = this.alphabeticalSort(this.forms.forms.lifecycleStages);
            this.forms.forms.emailSequenciesData = this.alphabeticalSort(this.forms.forms.emailSequencies);
            this.$el.html(_.template(FormRulesFormViewTmpl, {
                'forms': this.forms.forms.forms,
                'leadOwners': this.forms.forms.leadOwnersData,
                'currentUserId': this.forms.forms.currentUserId,
                'timezoneOffset': this.forms.forms.userTimezoneOffset,
                'moment': moment,
                'i18n': i18n,
                'userTimezone':this.forms.forms.userTimezone,
                'leadTags': this.forms.forms.leadTagsData,
                'lifecycleStages': this.forms.forms.lifecycleStagesData,
                'emailSequencies': this.forms.forms.emailSequenciesData
            }));

            $('#select-assign-lead-tags').chosen();
        },
        changeForm: function(e)
        {
            var el = $(e.currentTarget),
                formName = el.find(':selected').data('form-name'),
                formType = el.find(':selected').data('form-type'),
                self = this;

            if (el.val() == '0') {
                $('#form-field-selection-block').addClass('hidden');
                $('.form-field-rule-row-block').remove();
                $('#form-field-rules-configuration-block').addClass('hidden');
                return false;
            }

            if ($('#form-field-selection-block').hasClass('hidden')) {
                el.data('old-selection', el.val());
                self.getFormFields(formName, formType);
                $('#form-field-rules-configuration-block').addClass('hidden');
                $('.form-field-rule-row-block').remove();
            } else {
                showConfirm(_.isUndefined(i18n['Are you sure want to change the form? Form fields config will be updated']) ? 'Are you sure want to change the form? Form fields config will be updated' : i18n['Are you sure want to change the form? Form fields config will be updated'], function () {
                    self.getFormFields(formName, formType);
                    $('#form-field-rules-configuration-block').addClass('hidden');
                    $('.form-field-rule-row-block').remove();
                    el.data('old-selection', el.val());
                }, function(){
                    el.val(el.data('old-selection'));
                });
            }


        },
        switchTaskUnits: function(e)
        {
            if ($(e.currentTarget).val() == 'hour') {
                $('#lead-task-start-time').prop('disabled', true);
            } else {
                $('#lead-task-start-time').prop('disabled', false);
            }
        },
        switchImmediate: function(e)
        {
            var taskUnitsType = $('#form-action-assign-task-number-of-before-action-units').val();
            if ($(e.currentTarget).is(':checked')) {
                $('.disable-immediate').prop('disabled', true);
                $('#lead-task-start-time').prop('disabled', true);
            } else {
                $('.disable-immediate').prop('disabled', false);
                if (taskUnitsType === 'hour') {
                    $('#lead-task-start-time').prop('disabled', true);
                } else {
                    $('#lead-task-start-time').prop('disabled', false);
                }
            }
        },
        switcherAction:function (e) {
            var el      = $(e.currentTarget),
                tabId        = el.closest('li').find('a').data('tab-id'),
                currentTabEl = $('#'+tabId),
                actionType   = currentTabEl.data('action-type'),
                elSwitcher   = el.closest('li').find('input'),
                elSelect     = currentTabEl.find('select'),
                elInput      = currentTabEl.find('input'),
                elTextarea   = currentTabEl.find('textarea');

            this.disabledAction(elSwitcher, actionType, elSelect, elInput, elTextarea);
        },
        disabledAction: function (elSwitcher, actionType, elSelect, elInput, elTextarea) {
           var elTags  = $('#select_assign_lead_tags_chosen .chosen-results');

            if(elSwitcher.is(':checked')){
                if(actionType == 'assign_tags'){
                    elTags.css({'display' : ''});
                    if(elInput.length > 0 && elInput.attr('type') === 'checkbox'){
                        elInput.prop('disabled', false);
                    }
                }else{
                    if(elSelect.length > 0){
                        elSelect.prop('disabled', false);
                    }
                    if(elInput.length > 0){
                        elInput.prop('disabled', false);
                    }
                    if(elTextarea.length > 0){
                        elTextarea.prop('disabled', false);
                    }
                }
            }else {
                if(actionType == 'assign_tags'){
                    elTags.css({'display' : 'none'});
                    if(elInput.length > 0 && elInput.attr('type') === 'checkbox'){
                        elInput.prop('disabled', true);
                    }
                }else{
                    if(elSelect.length > 0){
                        elSelect.prop('disabled', true);
                    }
                    if(elInput.length > 0){
                        elInput.prop('disabled', true);
                    }
                    if(elTextarea.length > 0){
                        elTextarea.prop('disabled', true);
                    }
                }
            }
        },
        changeFormAction: function(e)
        {
            e.preventDefault();
            var el           = $(e.currentTarget),
                tabId        = el.data('tab-id'),
                currentTabEl = $('#'+tabId),
                actionType   = currentTabEl.data('action-type'),
                elSwitcher   = el.closest('li').find('input'),
                elSelect     = currentTabEl.find('select'),
                elInput      = currentTabEl.find('input'),
                elTextarea   = currentTabEl.find('textarea');

            this.disabledAction(elSwitcher, actionType, elSelect, elInput, elTextarea);

            $('#form-actions-list-block li').removeClass('current');
            $('.action-single-block').addClass('hidden');
            currentTabEl.removeClass('hidden');
            el.closest('li').addClass('current');
        },
        createLeadFormRule: function(e)
        {
            e.preventDefault();
            var ruleConfigBlocks = $('.form-field-rule-row-block'),
                actionSwitchers = $('#form-actions-list-block').find('input.switcher'),
                switchersChosen = false,
                error = false,
                fieldsConfigData = [],
                actionsData = [],
                self = this,
                formName = $('#forms-for-rules').find(':selected').data('form-name'),
                formType = $('#forms-for-rules').find(':selected').data('form-type'),
                formRuleName = $('#form-rule-name-general').val(),
                assignLeadOwnerId = $('#select-assign-lead-owner').val(),
                assignTaskBy = $('#form-action-assign-task-by').val(),
                assignTaskTo = $('#form-action-assign-task-to').val(),
                assignTaskNotes = $('#form-action-assign-task-notes').val(),
                assignTaskTitle = $('#form-action-assign-task-title').val(),
                assignTaskStartTime = $('#lead-task-start-time').val(),
                taskImmediate = 0,
                taskDelayUnits = $('#form-action-assign-task-number-of-before-action-units').val(),
                taskDelay= parseInt($('#form-action-assign-task-number-of-before-action').val()),
                taskTimezone = $('#user-task-timezone').val(),
                assignTaskStart = 0,
                userTimezoneOffset = $('#task-rule-user-timezone-offset').val(),
                tagIds =  $('#select-assign-lead-tags').val(),
                lifeCycleId = $('#select-assign-lead-lifecycle').val(),
                sequenceId = $('#select-assign-lead-sequence').val(),
                emailsToCopyRow = $('#email-copy-emails').val(),
                triggerNewLeadOnly = 0,
                unassignSequence = 0;

            if (ruleConfigBlocks.length === 0) {
                showMessage(_.isUndefined(i18n['Please create at least one rule']) ? 'Please create at least one rule' : i18n['Please create at least one rule'], true, false);
                return false;
            }

            $.each(ruleConfigBlocks, function(ruleConfigBlock) {
                fieldsConfigData.push({'field_name': $(this).data('field-name'), 'rule_comparison_operator' : $(this).find('.form-field-operator-value').val(), 'field_value' : $(this).find('.form-field-rules-row-value').val()});
            });

            $.each(actionSwitchers, function(switcher) {
                if ($(this).is(':checked') === true) {
                    var actionType = $(this).data('action-type');
                    switchersChosen = true;
                    if (actionType === 'assign_owner') {
                        if (assignLeadOwnerId == '0') {
                            showMessage(_.isUndefined(i18n['Please choose lead owner for the assign owner action']) ? 'Please choose lead owner for the assign owner action' : i18n['Please choose lead owner for the assign owner action'], true, false);
                            error = true;
                            return false;
                        } else {
                            triggerNewLeadOnly = 0;
                            if ($('#assign-lead-owner-only-for-new-lead').is(':checked')) {
                                triggerNewLeadOnly = 1;
                            }
                            actionsData.push({'assign_to' : assignLeadOwnerId, 'actionType': actionType, 'triggerNewLeadOnly': triggerNewLeadOnly});
                        }
                    }

                    if (actionType === 'assign_task') {
                        if (assignTaskTitle === '') {
                            showMessage(_.isUndefined(i18n['Please provide task title']) ? 'Please provide task title' : i18n['Please provide task title'], true, false);
                            error = true;
                            return false;
                        }

                        if ($('#form-action-assign-task-immediate').is(':checked')) {
                            taskImmediate = 1;
                        } else if (isNaN(taskDelay)) {
                            showMessage(_.isUndefined(i18n['Please enter task delay amount']) ? 'Please enter task delay amount' : i18n['Please enter task delay amount'], true, false);
                            error = true;
                            return false;
                        }

                        if (assignTaskStartTime.length == 1) {
                            assignTaskStart = '0' + assignTaskStartTime+':00:00';
                        } else {
                            assignTaskStart = assignTaskStartTime+':00:00';
                        }

                        triggerNewLeadOnly = 0;
                        if ($('#assign-task-only-for-new-lead').is(':checked')) {
                            triggerNewLeadOnly = 1;
                        }

                        actionsData.push({
                            'actionType': actionType,
                            'creator_id': assignTaskBy,
                            'user_id': assignTaskTo,
                            'notes': assignTaskNotes,
                            'title': assignTaskTitle,
                            'taskImmediate': taskImmediate,
                            'taskDelayUnits': taskDelayUnits,
                            'taskDelay': taskDelay,
                            'assignTaskTime': assignTaskStart,
                            'assignTaskStartTime': assignTaskStartTime,
                            'taskTimezone' : taskTimezone,
                            'userTimezoneOffset' : userTimezoneOffset,
                            'triggerNewLeadOnly' : triggerNewLeadOnly
                        });
                    }

                    if (actionType === 'assign_tags') {
                        if (tagIds == null) {
                            showMessage(_.isUndefined(i18n['Please choose at least one tag']) ? 'Please choose at least one tag' : i18n['Please choose at least one tag'], true, false);
                            error = true;
                            return false;
                        } else {
                            triggerNewLeadOnly = 0;
                            if ($('#assign-tags-only-for-new-lead').is(':checked')) {
                                triggerNewLeadOnly = 1;
                            }
                            actionsData.push({'tagIds' : tagIds, 'actionType': actionType, 'triggerNewLeadOnly':triggerNewLeadOnly});
                        }
                    }

                    if (actionType === 'assign_lifecycle') {
                        if (lifeCycleId == '0') {
                            showMessage(_.isUndefined(i18n['Please choose lifecycle']) ? 'Please choose lifecycle' : i18n['Please choose lifecycle'], true, false);
                            error = true;
                            return false;
                        } else {
                            triggerNewLeadOnly = 0;
                            if ($('#assign-lifecycle-only-for-new-lead').is(':checked')) {
                                triggerNewLeadOnly = 1;
                            }
                            actionsData.push({'lifecycleId' : lifeCycleId, 'actionType': actionType, 'triggerNewLeadOnly':triggerNewLeadOnly});
                        }
                    }
                    if (actionType === 'assign_sequence') {
                        if (sequenceId == '0') {
                            showMessage(_.isUndefined(i18n['Please choose email sequence']) ? 'Please choose email sequence' : i18n['Please choose email sequence'], true, false);
                            error = true;
                            return false;
                        } else {
                            triggerNewLeadOnly = 0;
                            if ($('#assign-sequence-only-for-new-lead').is(':checked')) {
                                triggerNewLeadOnly = 1;
                            }
                            unassignSequence = 0;
                            if ($('#unassign-sequence').is(':checked')) {
                                unassignSequence = 1;
                            }
                            actionsData.push({'sequenceId' : sequenceId, 'actionType': actionType, 'triggerNewLeadOnly':triggerNewLeadOnly, 'unassignSequence':unassignSequence});
                        }
                    }
                    if (actionType === 'assign_email_copy') {
                        var emailsToCopy = $.trim(emailsToCopyRow).split(',');
                        var emailValidation = /^(([^<>()\[\]\.,;:\s@\"]+(\.[^<>()\[\]\.,;:\s@\"]+)*)|(\".+\"))@(([^<>()[\]\.,;:\s@\"]+\.)+[^<>()[\]\.,;:\s@\"]{2,})$/i;
                        $.each(emailsToCopy, function(key, email){
                            email = $.trim(email);
                            var status = emailValidation.test(email);
                            if (status === false) {
                                var emailErrMsg = (_.isUndefined(i18n['The']) ? 'The' : i18n['The']) + ' ' + email + ' ' + (_.isUndefined(i18n['isn\'t valid.']) ? 'isn\'t valid.' : i18n['isn\'t valid.']) + ' ';
                                var validEmailAddressMsg = _.isUndefined(i18n['Please specify valid email address']) ? 'Please specify valid email address' : i18n['Please specify valid email address'];

                                showMessage(emailErrMsg + validEmailAddressMsg, true, false);
                                error = true;
                                return false;
                            }
                        });
                        if (_.isEmpty(emailsToCopy) || emailsToCopy === '') {
                            showMessage(_.isUndefined(i18n['Please specify email']) ? 'Please specify email' : i18n['Please specify email'], true, false);
                            error = true;
                            return false;
                        } else {
                            triggerNewLeadOnly = 0;
                            if ($('#assign-email-copy-only-for-new-lead').is(':checked')) {
                                triggerNewLeadOnly = 1;
                            }
                            actionsData.push({'emailsToCopy' : emailsToCopyRow, 'actionType': actionType, 'triggerNewLeadOnly':triggerNewLeadOnly});
                        }
                    }
                }
            });

            if (error === true) {
                return false;
            }

            if (switchersChosen === false) {
                showMessage(_.isUndefined(i18n['Please activate at least one action']) ? 'Please activate at least one action' : i18n['Please activate at least one action'], true, false);
                return false;
            }

            if (formRuleName === '') {
                showMessage(_.isUndefined(i18n['Please specify rule name']) ? 'Please specify rule name' : i18n['Please specify rule name'], true, false);
                return false;
            }

           var data = {'secureToken': $('#form-leads-grid-token').val(), 'rule_name': formRuleName, 'form_name': formName, 'form_type': formType, 'actionsData' :actionsData, 'fieldsData' : fieldsConfigData};

            $.ajax({
                url: $('#website_url').val()+'api/leads/leadsformrules',
                type: 'POST',
                data: data,
                dataType: 'json'
            }).done(function(response) {
                self.renderConfigForm();
                self.$el.trigger('LeadFormRulesFormGrid:created');
                showMessage(response.responseText.message, false, 5000);
            }).fail(function(response){
                showMessage(response.responseJSON, true, 5000);
            });
            return false;
        },
        changeRuleOperator: function(e)
        {
            var el = $(e.currentTarget),
                ruleOperator = el.val(),
                valueEl = el.closest('div').find('.form-field-rules-row-value'),
                infoTooltip = window.appLeadFormRouter.leadTooltipInfo(ruleOperator);

            el.prop('title', infoTooltip);

            if (ruleOperator === 'in') {
                valueEl.replaceWith('<textarea placeholder="'+(_.isUndefined(i18n['separate the values with commas']) ? 'separate the values with commas' : i18n['separate the values with commas'])+'" class="form-field-rules-row-value grid_5 alpha omega" name="'+valueEl.prop('name')+'">'+valueEl.val()+'</textarea>');
            } else if(!valueEl.is('input')) {
                valueEl.replaceWith('<input type="text" class="form-field-rules-row-value grid_5 alpha omega" name="'+valueEl.prop('name')+'" value="'+valueEl.val()+'" />');
            }
        },
        getFormFields:  function(formName, formType)
        {
            $.ajax({
                'url' : $('#website_url').val()+'plugin/leads/run/getFormFieldsList',
                'type' : 'GET',
                'dataType' : 'json',
                'data' : {'formName' : formName, 'formType': formType}
            }).done(function(response){
                if (response.error == '0') {
                    $('#form-field-selection').empty();
                    var options = '';
                    $.each(response.responseText.formFields, function(formField){
                        options += '<option value="'+formField+'">'+formField+'</option>';
                    });
                    $('#form-field-selection').html(options);
                    $('#form-field-selection-block').removeClass('hidden');

                } else {
                    showMessage(response.responseText, true, 5000);
                }
            });

        },
        addField: function()
        {
           var fieldName = $('#form-field-selection').val(),
               fieldKey = fieldName.replace(/\s/g, '');

           $('#form-field-rules-configuration-block').removeClass('hidden');
           if ($('#form-field-rule-row-block-'+fieldKey).length == '0') {
               $('#form-field-rules-configuration-block-rows').append( _.template(FormRulesFieldRowTmpl, {'fieldKey': fieldKey, 'fieldName': fieldName, 'moment': moment, 'i18n':i18n}));
           } else {
               showMessage(_.isUndefined(i18n['Field has already added']) ? 'Field has already added' : i18n['Field has already added'], true, 5000);
           }


        },
        deleteFieldRow: function(e)
        {
            e.preventDefault();
            showConfirm(_.isUndefined(i18n['Are you sure want to delete?']) ? 'Are you sure want to delete?' : i18n['Are you sure want to delete?'], function () {
                $(e.currentTarget).closest('.form-field-rule-row-block').remove();
            });

        },
        alphabeticalSort: function(obj){
            // convert object into array
            var sortable=[];
            for(var key in obj)
                if(obj.hasOwnProperty(key))
                    sortable.push([key, obj[key]]); // each item is an array in format [key, value]

            // sort items by value
            sortable.sort(function(a, b)
            {
                var x=a[1].toLowerCase(),
                    y=b[1].toLowerCase();
                return x<y ? -1 : x>y ? 1 : 0;
            });
            return sortable; // array in format [ [ key1, val1 ], [ key2, val2 ], ... ]
        }
    });

    return LeadFormRulesView;
});