define([
    'backbone',
    'i18n!../../../nls/dashboard_'+$('#main-dashboard-system-language').val()+'_ln'
], function(Backbone, i18n){
    var LeadFilterPresetView = Backbone.View.extend({
        el: $('#filter-settings-preset-block'),
        events: {
            'click #save-filter-preset': 'saveFilterPreset',
            'change #lead-predefined-filter-list': 'changeFilterPreset'
        },
        templates: {

        },
        initialize: function(){
            this.$el = $('#filter-settings-preset-block');
        },
        render: function(){
            return this;
        },
        saveFilterPreset: function(e)
        {
            e.preventDefault();
            var leadFilterPresetName = $('#lead-filter-preset-name').val(),
                leadFilterPresetDefault = 0,
                leadFilterAllowPreset = 'individual',
                requestType = 'POST',
                leadPresetId = $('#lead-preset-id').val(),
                filterPresetData = {};
            if (!_.isEmpty(leadFilterPresetName)) {
                if ($('#lead-filter-preset-default').is(':checked')) {
                    leadFilterPresetDefault = 1;
                }
                if ($('#lead-filter-preset-allow').is(':checked')) {
                    leadFilterAllowPreset = 'all';
                }
                if (!_.isEmpty(leadPresetId)) {
                    requestType = 'PUT';
                }

                filterPresetData = {'created_at_from' : ($('#lead-filter-from-date').val()) ? $.datepicker.formatDate('yy-mm-dd', $('#lead-filter-from-date').datepicker("getDate")): '',
                    'created_at_to': ($('#lead-filter-to-date').val()) ? $.datepicker.formatDate('yy-mm-dd', $('#lead-filter-to-date').datepicker("getDate")): '',
                    'last_activity_from': ($('#lead-filter-activity-date').val()) ? $.datepicker.formatDate('yy-mm-dd', $('#lead-filter-activity-date').datepicker("getDate")): '',
                    'last_activity_to': ($('#lead-filter-activity-to-date').val()) ? $.datepicker.formatDate('yy-mm-dd', $('#lead-filter-activity-to-date').datepicker("getDate")): '',
                    'stage_id': $('#lead-filter-by-stage').val(),
                    'customer_group_id':$('#lead-filter-by-group').val(),
                    'lead_source_type': $('#lead-filter-by-source-type').val(),
                    'lead_source': $('#lead-filter-by-source').val(),
                    'owner_id': $('#lead-filter-by-owner').val(),
                    'lead_zip': $('#lead-filter-zip').val(),
                    'lead_country': $('#lead-filter-country').val(),
                    'lead_city': $('#lead-filter-city').val(),
                    'lead_filter_total_opportunity_amount_from': $('#lead-filter-total-opportunity-amount-from').val(),
                    'lead_filter_total_opportunity_amount_to': $('#lead-filter-total-opportunity-amount-to').val(),
                    'special_filter_type': $('input[name=leadFilterBySpecialType]:checked').val(),
                    'created_at': $('#switch-created-date:checked').val(),
                    'last_updated': $('#switch-updated-date:checked').val(),
                    'lead_tags': $('#lead-filter-tag').val(),
                    'lead_opportunity_stages': $('#lead-filter-opportunity-stage').val(),
                    'last_interaction_id':$('#lead-filter-by-last-interaction').val(),
                    'opportunity_type_id':$('#lead-filter-by-opportunity-type').val(),
                    'lead_task':$('#lead-filter-task').val()
                };

                var formParams = {'filter_preset_type': $('#filter-preset-type').val(), 'filter_preset_name':leadFilterPresetName,'is_default': leadFilterPresetDefault,
                    'access': leadFilterAllowPreset, 'filter_preset_data': filterPresetData, 'secureToken': $('#lead-grid-token').val()
                    };
                if (requestType === 'PUT') {
                    formParams.id = leadPresetId;
                    formParams = JSON.stringify(formParams);
                }

                $.ajax({
                    'url': $('#website_url').val() + 'api/leads/leadfilterpreset/',
                    'type' : requestType,
                    'dataType': 'json',
                    'data': formParams
                }).done(function(response){
                    $('#lead-preset-id').val('');
                    $('#lead-filter-preset-default').prop('checked', false);
                    $('#lead-filter-preset-allow').prop('checked', false);
                    $('#lead-filter-preset-name').val('');
                    if (requestType === 'POST') {
                        $('#lead-predefined-filter-list').append('<option value="' + response.responseText.id + '">' + leadFilterPresetName + '</option>');
                    } else {
                        $('#lead-predefined-filter-list option:selected').text(leadFilterPresetName);
                    }
                    $('#lead-predefined-filter-list').val(0);
                    showMessage(response.responseText.message, false, 5000);
                    $('#lead-filter-reset').trigger('click');
                }).fail(function(response) {
                    showMessage(response.responseJSON, true, 5000);
                });
            }
        },
        changeFilterPreset:function(e)
        {
            var presetId = $(e.currentTarget).val();
            if (_.isEmpty(presetId)) {
                return false;
            }

            if (presetId == '0') {
                $('#lead-preset-id').val('');
                $('#lead-filter-reset').trigger('click');
                return false;
            }

            $.ajax({
                'url': $('#website_url').val()+'api/leads/leadfilterpreset/',
                'type':'GET',
                'dataType':'json',
                'data': {'id': presetId, 'secureToken': $('#lead-grid-token').val()}
            }).done(function(responseData){
                if (typeof responseData.isDefault !=='undefined' && responseData.isDefault == '1') {
                    $('#lead-filter-preset-default').prop('checked', true);
                } else {
                    $('#lead-filter-preset-default').prop('checked', false);
                }
                if (typeof responseData.access !=='undefined' && responseData.access === 'individual') {
                    $('#lead-filter-preset-allow').prop('checked', false);
                } else {
                    $('#lead-filter-preset-allow').prop('checked', true);
                }
                var filtersData = JSON.parse(responseData.filterPresetData),
                    fromDate = '',
                    toDate = '',
                    activityFrom = '',
                    activityTo = '';

                if (filtersData.created_at_from !== '') {
                    fromDate = $.datepicker.formatDate('d-M-yy', new Date(filtersData.created_at_from));
                }
                if (filtersData.created_at_to !== '') {
                    toDate = $.datepicker.formatDate('d-M-yy', new Date(filtersData.created_at_to));
                }
                if (filtersData.last_activity_from !== '') {
                    activityFrom = $.datepicker.formatDate('d-M-yy', new Date(filtersData.last_activity_from));
                }
                if (filtersData.last_activity_to !== '') {
                    activityTo = $.datepicker.formatDate('d-M-yy', new Date(filtersData.last_activity_to));
                }

                $('#lead-filter-from-date').val(fromDate);
                $('#lead-filter-to-date').val(toDate);
                $('#lead-filter-activity-date').val(activityFrom);
                $('#lead-filter-activity-to-date').val(activityTo);
                $('#lead-filter-by-stage').val(filtersData.stage_id);
                $('#lead-filter-by-group').val(filtersData.customer_group_id);
                $('#lead-filter-by-source').val(filtersData.lead_source);
                $('#lead-filter-total-opportunity-amount').val(filtersData.total_opportunity_amount);
                $('#lead-filter-by-owner').val(filtersData.owner_id);
                $('#lead-filter-city').val(filtersData.lead_city);
                $('#lead-filter-zip').val(filtersData.lead_zip);
                if (!_.isUndefined(filtersData.lead_task) && !_.isEmpty(filtersData.lead_task)) {
                    $('#lead-filter-task').val(filtersData.lead_task);
                } else {
                    $('#lead-filter-task').val(0);
                }

                if (!_.isUndefined(filtersData.lead_source_type) && !_.isEmpty(filtersData.lead_source_type)) {
                    $('#lead-filter-by-source-type').val(filtersData.lead_source_type);
                } else {
                    $('#lead-filter-by-source-type').val(0);
                }

                $('#lead-filter-total-opportunity-amount-from').val(filtersData.lead_filter_total_opportunity_amount_from);
                $('#lead-filter-total-opportunity-amount-to').val(filtersData.lead_filter_total_opportunity_amount_to);
                if (filtersData.lead_country) {
                    $('#lead-filter-country').val(filtersData.lead_country);
                } else {
                    $('#lead-filter-country').val(0);
                }
                $('#lead-filter-preset-name').val(responseData.filterPresetName);
                $('#lead-preset-id').val(presetId);

                if (!_.isUndefined(filtersData.special_filter_type) && filtersData.special_filter_type === 'existing_client') {
                    $('#lead-filter-by-special-type-existing-client').prop('checked', true);
                } else  if (!_.isUndefined(filtersData.special_filter_type) && filtersData.special_filter_type === 'prospect') {
                    $('#lead-filter-by-special-type-prospect').prop('checked', true);
                } else {
                    $('#lead-filter-by-special-type-existing-client').prop('checked', false);
                    $('#lead-filter-by-special-type-prospect').prop('checked', false);
                }

                if (filtersData.last_interaction_id) {
                    $('#lead-filter-by-last-interaction').val(filtersData.last_interaction_id);
                } else {
                    $('#lead-filter-by-last-interaction').val(0);
                }

                if (filtersData.opportunity_type_id) {
                    $('#lead-filter-by-opportunity-type').val(filtersData.opportunity_type_id);
                } else {
                    $('#lead-filter-by-opportunity-type').val(0);
                }

                if (!_.isUndefined(filtersData.last_updated) && filtersData.last_updated === 'last_updated') {
                    $('#switch-updated-date').prop('checked', true);
                } else {
                    $('#switch-updated-date').prop('checked', false);
                }

                if (!_.isUndefined(filtersData.created_at) && filtersData.created_at === 'created_at') {
                    $('#switch-created-date').prop('checked', true);
                } else {
                    $('#switch-created-date').prop('checked', false);
                }

                if (!_.isUndefined(filtersData.lead_tags) && !_.isEmpty(filtersData.lead_tags)) {
                    $('#lead-filter-tag').val(filtersData.lead_tags).trigger("chosen:updated");
                } else {
                    $('#lead-filter-tag').val(0).trigger("chosen:updated");
                }

                if (!_.isUndefined(filtersData.lead_opportunity_stages) && !_.isEmpty(filtersData.lead_opportunity_stages)) {
                    $('#lead-filter-opportunity-stage').val(filtersData.lead_opportunity_stages).trigger("chosen:updated");
                } else {
                    $('#lead-filter-opportunity-stage').val(0).trigger("chosen:updated");
                }


                $('#lead-filter-apply').trigger('click');
            }).fail(function(response) {
                showMessage(response.responseJSON, true, 5000);
            });
        }
    });

    return LeadFilterPresetView;
});