define([
    'backbone',
    '../collections/leads',
    'text!../templates/paginator.html',
    'text!../templates/callWindow.html',
    'text!../templates/lead-grid-row.html',
    'text!../templates/lead-action-assign-owner.html',
    'text!../templates/lead-action-email-service.html',
    'i18n!../../../nls/dashboard_'+$('#main-dashboard-system-language').val()+'_ln',
    'moment',
    'chosen',
    'accounting',
    'text!../templates/lead-export-dialog.html',
    'text!../templates/lead-action-email-sequence.html',
    'text!../templates/lead-action-assign-tags.html'
], function(Backbone,
            LeadsCollection, PaginatorTmpl, CallDialogTmpl, LeadGridRowTmpl, LeadActionAssignOwnerTemplate,
            LeadActionEmailServiceTemplate, i18n, moment, chosen, accounting, leadExportDialog,
            leadActionEmailSequenceTemplate, leadActionAssignTagsTemplate
){

    var LeadsGridView = Backbone.View.extend({
        el: $('#lead-grid-table'),
        events: {
            'click a[data-role=delete]': 'deleteLead',
            'click a[data-role=call]': 'callLeadDialog',
            'click a[data-role=email]': 'displayEmailTab',
            'click td.lead-grid-paginator a.page': 'navigate',
            'blur input.lead-edit-attribute': 'editLeadAttribute',
            'change select.lead-edit-attribute': 'editLeadAttribute',
            'change input.check-lead-in-grid': 'checkLeadInGrid',
            'change #leads-mass-action': 'massAction',
            'change input.check-all-leads' :'checkAllInGrid'
        },
        templates: {
            paginator: _.template(PaginatorTmpl)
        },
        initialize: function(options){
            this.$el.unbind();

            if (!_.isEmpty(options)) {
              //  $('#lead-filter-opportunity-stage').val(options['leadFilterOpportunityStage']);
            }
            this.leads = new LeadsCollection();
            this.leads.isGrid = true;

            this.leads.on('reset', this.renderLeads, this);
            this.leads.on('add', this.renderLeads, this);
            this.leads.on('destroy', this.renderLeads, this);

            var filterFromDate = $('#lead-filter-from-date'),
                filterToDate = $('#lead-filter-to-date'),
                filterActivityFrom = $('#lead-filter-activity-date'),
                filterActivityTo = $('#lead-filter-activity-to-date');

            filterFromDate.datepicker({
                dateFormat: 'd-M-yy',
                defaultDate: "+1w",
                changeMonth: true,
                changeYear: true,
                yearRange: "c-5:c+5",
                onSelect: function(selectedDate){
                    filterToDate.datepicker("option", "minDate", selectedDate);
                }
            });
            filterToDate.datepicker({
                dateFormat: 'd-M-yy',
                defaultDate: "+1w",
                changeMonth: true,
                changeYear: true,
                yearRange: "c-5:c+5",
                onSelect: function(selectedDate){
                    filterFromDate.datepicker("option", "maxDate", selectedDate);
                }
            });
            filterActivityFrom.datepicker({
                dateFormat: 'd-M-yy',
                defaultDate: "+1w",
                changeMonth: true,
                changeYear: true,
                yearRange: "c-5:c+5",
                minDate: +1,
                onSelect: function(selectedDate){
                    filterActivityTo.datepicker("option", "minDate", selectedDate);
                }
            });

            filterActivityTo.datepicker({
                dateFormat: 'd-M-yy',
                defaultDate: "+1w",
                changeMonth: true,
                changeYear: true,
                yearRange: "c-5:c+5",
                onSelect: function(selectedDate){
                    filterActivityFrom.datepicker("option", "maxDate", selectedDate);
                }
            });

            $('#lead-filter-tag').chosen();
            $('#lead-filter-opportunity-stage').chosen();
        },
        render: function(){
            this.leads.server_api = _.extend(this.leads.server_api, {
                'filter': function() {
                    return {
                        'filter-by': $('input[name=searchByKeyword]:checked').val(),
                        'lead-attribute': $('#lead-attribute').val(),
                        'created_at_from' : ($('#lead-filter-from-date').val()) ? $.datepicker.formatDate('yy-mm-dd', $('#lead-filter-from-date').datepicker("getDate")): '',
                        'created_at_to': ($('#lead-filter-to-date').val()) ? $.datepicker.formatDate('yy-mm-dd', $('#lead-filter-to-date').datepicker("getDate")): '',
                        'next_activity_from': ($('#lead-filter-activity-date').val()) ? $.datepicker.formatDate('yy-mm-dd', $('#lead-filter-activity-date').datepicker("getDate")): '',
                        'next_activity_to': ($('#lead-filter-activity-to-date').val()) ? $.datepicker.formatDate('yy-mm-dd', $('#lead-filter-activity-to-date').datepicker("getDate")): '',
                        'stage_id': $('#lead-filter-by-stage').val(),
                        'customer_group_id':$('#lead-filter-by-group').val(),
                        'lead_source_type': $('#lead-filter-by-source-type').val(),
                        'lead_source': $('#lead-filter-by-source').val(),
                        'owner_id': $('#lead-filter-by-owner').val(),
                        'lead_zip': $('#lead-filter-zip').val(),
                        'lead_country': $('#lead-filter-country').val(),
                        'lead_city': $('#lead-filter-city').val(),
                        'total_opportunity_amount_from': $('#lead-filter-total-opportunity-amount-from').val(),
                        'total_opportunity_amount_to': $('#lead-filter-total-opportunity-amount-to').val(),
                        'special_filter_type': $('input[name=leadFilterBySpecialType]:checked').val(),
                        'created_at': $('#switch-created-date:checked').val(),
                        'last_updated': $('#switch-updated-date:checked').val(),
                        'lead_tags': $('#lead-filter-tag').val(),
                        'lead_opportunity_stages': $('#lead-filter-opportunity-stage').val(),
                        'last_interaction_id':$('#lead-filter-by-last-interaction').val(),
                        'opportunity_type_id':$('#lead-filter-by-opportunity-type').val(),
                        'interaction_value':$('#lead-filter-by-interaction').val(),
                        'lead_task':$('#lead-filter-task').val()
                    };
                }
            });
            this.leads.pager();
        },
        renderLeads: function(){
            this.$el.find('#lead-grid-table-body').empty();
            this.leads.each(this.renderLead, this);
            this.leads.info()['i18n'] = i18n;
            this.$('td.lead-grid-paginator').html(this.templates.paginator(this.leads.information));
            if (typeof tableResponsive === "function") {
                tableResponsive();
            }
            this.assignTotalOpportunitiesAmount(this.leads.pluck("id"));

            var self = this;
            if (!_.isUndefined(this.checkedLeadRows)) {
                _.each($('input.check-lead-in-grid'), function (el) {
                    if (self.checkedLeadRows.indexOf($(el).data('lead-id')) !== -1) {
                        $(el).prop('checked', true);
                    }
                });
            }

            $('input.check-all-leads').prop('checked', false);

            $(".lead-edit-organization").on("keydown", function(event) {
                if ( event.keyCode === $.ui.keyCode.TAB &&
                    $(this).autocomplete( "instance" ).menu.active) {
                    event.preventDefault();
                }
            }).autocomplete({
                    source: function(request, response) {
                        $.ajax({
                            'url': $('#website_url').val()+'api/leads/leadorganization/',
                            'type':'GET',
                            'dataType':'json',
                            'data': {searchTerm: request.term}
                        }).done(function(responseData){
                            if (!_.isEmpty(responseData)) {
                                response($.map(responseData, function (responseData) {
                                    return {
                                        label: responseData.organization_name,
                                        value: responseData.organization_name
                                    };
                                }));
                            }
                        });
                    },
                    search: function() {

                    },
                    focus: function() {
                        return false;
                    },
                    select: function(event, ui) {

                    },
                    messages: {
                        noResults: '',
                        results: function() {}
                    }
            });

            $(".lead-edit-source").on("keydown", function(event) {
                if ( event.keyCode === $.ui.keyCode.TAB &&
                    $(this).autocomplete("instance").menu.active) {
                    event.preventDefault();
                }
            }).autocomplete({
                source: function(request, response) {
                    $.ajax({
                        'url': $('#website_url').val()+'api/leads/leadsource/',
                        'type':'GET',
                        'dataType':'json',
                        'data': {searchTerm: request.term, 'autocomplete':true}
                    }).done(function(responseData){
                        if (!_.isEmpty(responseData)) {
                            response($.map(responseData, function (responseData) {
                                return {
                                    label: responseData.source_name,
                                    value: responseData.source_name
                                };
                            }));
                        }
                    });
                },
                search: function() {

                },
                focus: function() {
                    return false;
                },
                select: function(event, ui) {

                }
            });
        },
        renderLead: function(leadsData){
            this.leads.customerGroupsData = this.alphabeticalSort(this.leads.customerGroups);
            this.leads.leadStagesData = this.alphabeticalSort(this.leads.leadStages);
            this.leads.leadTypesData = this.alphabeticalSort(this.leads.leadTypes);
            this.$el.find('#lead-grid-table-body').append(_.template(LeadGridRowTmpl, {'leadModel': leadsData,
                'customerGroups': this.leads.customerGroupsData, 'stagesList' : this.leads.leadStagesData, 'leadTypeList': this.leads.leadTypesData, 'interactionParam': this.leads.interactionParam, 'moment':moment, 'i18n': i18n}));

            var desktopMasks = this.leads.desktopMasks,
                desktopCountryCode = leadsData.get('lead_phone_country_code');

            if (typeof desktopMasks[desktopCountryCode] !== 'undefined') {
                $('#lead-edit-attribute-phone-'+leadsData.get('id')).mask(desktopMasks[desktopCountryCode].mask_value, {autoclear: false});
            } else {
                $('#lead-edit-attribute-phone-'+leadsData.get('id')).mask('(999) 999 9999', {autoclear: false});
            }
        },
        checkAllInGrid: function(e) {
            var allEl = $(e.currentTarget),
                currentState = false,
                self = this;
            if (allEl.is(':checked')) {
                currentState = true;
            }

            _.each($('input.check-lead-in-grid'), function (el) {
                $(el).prop('checked', currentState);
                 if (currentState === true) {
                    if (!_.isUndefined(self.checkedLeadRows)) {
                        self.checkedLeadRows = _.union(self.checkedLeadRows, $(el).data('lead-id'));
                    } else {
                        self.checkedLeadRows = [$(el).data('lead-id')];
                    }
                }  else {
                    if (!_.isUndefined(self.checkedLeadRows)) {
                        var arrayIndex = self.checkedLeadRows.indexOf($(el).data('lead-id'));
                        self.checkedLeadRows.splice(arrayIndex, 1);
                    }
                }
            });
        },
        alphabeticalSort: function(obj){
            // convert object into array
            var sortable=[];
            for(var key in obj)
                if(obj.hasOwnProperty(key))
                    sortable.push([key, obj[key]]); // each item is an array in format [key, value]

            // sort items by value
            sortable.sort(function(a, b)
            {
                var x=a[1].toLowerCase(),
                    y=b[1].toLowerCase();
                return x<y ? -1 : x>y ? 1 : 0;
            });
            return sortable; // array in format [ [ key1, val1 ], [ key2, val2 ], ... ]
        },
        checkLeadInGrid:function(e)
        {
            var el = $(e.currentTarget),
                leadId = el.data('lead-id'),
                status = el.is(':checked');

            if (status === true) {
                if (!_.isUndefined(this.checkedLeadRows)) {
                    this.checkedLeadRows = _.union(this.checkedLeadRows, leadId);
                } else {
                    this.checkedLeadRows = [leadId];
                }
            } else {
                if (!_.isUndefined(this.checkedLeadRows)) {
                    var arrayIndex = this.checkedLeadRows.indexOf(leadId);
                    this.checkedLeadRows.splice(arrayIndex, 1);
                }
            }
        },
        massAction: function(e){
            var func = $(e.currentTarget).val()+'Action';

            if (_.isFunction(this[func])) {
                var leads = this.checkedLeadRows,
                    self = this;
                if (!_.isUndefined(leads)){
                    this[func].call(self, leads);
                }
            }
            $(e.currentTarget).val(0);
        },
        addTagsAction: function () {
            var self = this,
                checkedLeads = this.checkedLeadRows.join(','),
                countSelectedLeads = this.checkedLeadRows.length;

            if (!_.isEmpty(this.checkedLeadRows)) {
                $.ajax({
                    url: $('#website_url').val() + 'plugin/leads/run/addTags/leadsIds/' + checkedLeads,
                    type: 'GET',
                    dataType: 'json'

                }).done(function (response) {
                    var tags = self.alphabeticalSort(response.tags),
                        selectedTags = response.selectedTags,
                        emptyTagsMsg = '';

                    if(_.isEmpty(tags)){
                        emptyTagsMsg = _.isUndefined(i18n['You don\'t have any saved tags. Please add tags in global config.']) ? 'You don\'t have any saved tags. Please add tags in global config.' : i18n['You don\'t have any saved tags. Please add tags in global config.'];
                    }

                    var dialog = _.template(leadActionAssignTagsTemplate, {
                        leadIds: checkedLeads,
                        countSelectedLeads: countSelectedLeads,
                        i18n: i18n,
                        tags: tags,
                        selectedTags: selectedTags,
                        emptyTagsMsg: emptyTagsMsg,
                        maxheight: ($(window).height() - $(window).height() * 0.1)- 180
                    });
                    $(dialog).dialog({
                        dialogClass: 'seotoaster',
                        width: '40%',
                        maxHeight: ($(window).height() - $(window).height() * 0.1),
                        resizable: false,
                        open: function (event, ui) {
                            $('#assign-tags').on('submit', function (e) {
                                e.preventDefault();
                                var checkedTagsIds = [],
                                    self = this;
                                $('input[name=lead-tags]:checked').map(function () {
                                    checkedTagsIds.push($(this).val());
                                });

                                var tagsFunction = $('input[name=tags-function]:checked').val(),
                                    tagsMsg = 'Save changes? All tags will be reprocessed!';

                                if(tagsFunction == 'addnew'){
                                    tagsMsg = 'Save changes? Selected tag(s) will be added to lead(s)!';
                                }

                                showConfirm(_.isUndefined(i18n[tagsMsg])?tagsMsg:i18n[tagsMsg], function(){
                                    $.ajax({
                                        url: $(self).attr('action'),
                                        type: 'POST',
                                        dataType: 'json',
                                        data: {
                                            'secureToken' : $('#lead-grid-token').val(),
                                            'tagsIds'     : checkedTagsIds,
                                            'leadIds'     : checkedLeads,
                                            'tagsFunction': tagsFunction
                                        }
                                    }).done(function (response) {
                                        showMessage(response.responseText, false, 5000);
                                        $('#assign-tags-dialog').dialog('close');
                                    });
                                });
                                $('.error').css({'z-index': 10000});
                                $('.ui-dialog').css({'z-index': 99});
                            });
                        },
                        close: function (event, ui) {
                            $(this).dialog('close').remove();
                        }
                    });
                    return false;
                });
            }
            return false;
        },
        emailSequenceAction:function (e) {
            var self = this,
                checkedLeads = this.checkedLeadRows.join(','),
                countSelectedLeads = this.checkedLeadRows.length;

            $.ajax({
                url: $('#website_url').val()+'plugin/leads/run/emailSequence/',
                type: 'GET',
                dataType: 'json'

            }).done(function(response) {
                var emailSequences = self.alphabeticalSort(response.emailSequences);
                var dialog = _.template(leadActionEmailSequenceTemplate, {
                    leadIds: checkedLeads,
                    countSelectedLeads: countSelectedLeads,
                    i18n:i18n,
                    emailSequences: emailSequences
                });
                $(dialog).dialog({
                    dialogClass: 'seotoaster',
                    width: '30%',
                    height: '200',
                    resizable: false,
                    open: function(event, ui) {
                        $('#assign-email-sequence').on('submit',  function(e){
                            e.preventDefault();
                            var leadEmailSequenceId = $(this).find('.lead-email-sequences').val();

                            if (leadEmailSequenceId == '0')  {
                                showMessage(_.isUndefined(i18n['Please choose email sequence']) ? 'Please choose email sequence' : i18n['Please choose email sequence'], true, 5000);
                                $('.error').css({'z-index': 10000});
                                return false;
                            }

                            $.ajax({
                                url: $(this).attr('action'),
                                type: 'POST',
                                dataType: 'json',
                                data:{'secureToken' : $('#lead-grid-token').val(), 'leadEmailSequenceId' : leadEmailSequenceId, 'leadIds' : checkedLeads}
                            }).done(function(response){
                                if (response.error == '1') {
                                    showMessage(response.responseText, true, 5000);
                                } else {
                                    showMessage(response.responseText, false, 5000);
                                    $('#assign-email-sequence-dialog').dialog('close');
                                }
                            });
                        });
                    },
                    close: function(event, ui){
                        $(this).dialog('close').remove();
                    }
                });

                return false;
            });
        },
        exportLeadsAction: function () {
            var leadIds = this.checkedLeadRows.join(','),
                countSelectedLeads = this.checkedLeadRows.length,
                filters = this.leads.server_api.filter();

            $.ajax({
                url: $('#website_url').val()+'plugin/leads/run/getExportLeadsTabData/',
                type: 'GET',
                dataType: 'json'

            }).done(function(response) {
                var dialog = _.template(leadExportDialog, {
                    leadIds: leadIds,
                    filters : $.param(filters),
                    countSelectedLeads: countSelectedLeads,
                    i18n:i18n,
                    'defaultConfig': response.responseText.defaultConfig,
                    'additionalLeadsExportConfig': response.responseText.additionalLeadsExportConfig
                });
                $(dialog).dialog({
                    dialogClass: 'seotoaster',
                    width: '75%',
                    height: '750',
                    resizable: false,
                    open: function(event, ui) {
                        $('.allLeads').on('change',  function(e){
                            e.preventDefault();

                            var exportLeadsText = _.isUndefined(i18n['Export leads']) ? 'Export leads':i18n['Export leads'],
                                totalText = _.isUndefined(i18n['Total'])?'Total':i18n['Total'],
                                recordsText = _.isUndefined(i18n['records'])?'records':i18n['records'],
                                title = '';

                            if($('.allLeads').prop('checked')){
                                $.ajax({
                                    url: $('#website_url').val()+'plugin/leads/run/countAllLeads/',
                                    type: 'POST',
                                    dataType: 'json',
                                    data:{'secureToken':$('#lead-grid-token').val(), 'filters':$.param(filters)}
                                }).done(function(response){
                                    if (response.error == '1') {
                                        showMessage(_.isUndefined(i18n['No leads found'])?'No leads found':i18n['No leads found'], true, 5000);
                                    } else {
                                        var countSelectedLeadsQuantity = response.responseText.quantity,
                                            title = exportLeadsText + ' (' + totalText +' '+ countSelectedLeadsQuantity +' '+ recordsText +')';

                                       $('.export-lead-dialog').dialog('option', 'title', title);
                                    }
                                });
                            }else{
                                title = exportLeadsText + ' (' + totalText +' '+ countSelectedLeads +' '+ recordsText +')';
                                $('.export-lead-dialog').dialog('option', 'title', title);
                            }
                        });
                    },
                    close: function(event, ui){
                        $(this).dialog('close').remove();
                    }
                });

                return false;
            });
        },
        changeLeadOwnerAction: function()
        {
            var checkedLeads = this.checkedLeadRows.join(','),
                countSelectedLeads = this.checkedLeadRows.length;

            $.ajax({
                url: $('#website_url').val()+'plugin/leads/run/getTaskTabData/',
                type: 'GET',
                dataType: 'json'

            }).done(function(response) {
                var dialog = _.template(LeadActionAssignOwnerTemplate, {
                    leadIds: checkedLeads,
                    countSelectedLeads: countSelectedLeads,
                    i18n:i18n,
                    'leadOwners': response.owners
                });
                $(dialog).dialog({
                    dialogClass: 'seotoaster',
                    width: '30%',
                    height: '200',
                    resizable: false,
                    open: function(event, ui) {
                        $('#assign-lead-owner').on('submit',  function(e){
                            e.preventDefault();
                            var leadOwnerId = $(this).find('.lead-owners').val();

                            if (leadOwnerId == '0' || leadOwnerId == null)  {
                                showMessage(_.isUndefined(i18n['Please choose lead owner'])?'Please choose lead owner':i18n['Please choose lead owner'], true, 5000);
                                $('.error').css({'z-index': 10000});
                                return false;
                            }

                            $.ajax({
                                url: $(this).attr('action'),
                                type: 'POST',
                                dataType: 'json',
                                data:{'secureToken':$('#lead-grid-token').val(), 'leadOwnerId': leadOwnerId, 'leadIds':checkedLeads}
                            }).done(function(response){
                                if (response.error == '1') {
                                    showMessage(response.responseText, true, 5000);
                                } else {
                                    showMessage(response.responseText, false, 5000);
                                    $('#assign-lead-owner-dialog').dialog('close');
                                }
                            });
                        });
                    },
                    close: function(event, ui){
                        $(this).dialog('close').remove();
                    }
                });

                return false;
            });
        },
        emailMarketingAction: function(e) {
            var checkedLeads = this.checkedLeadRows.join(','),
                customerIds = [1];

            $.ajax({
                url: $('#website_url').val()+'plugin/apps/run/getEnabledServicesDashboard/serviceType/email/customers/'+customerIds,
                type: 'GET',
                dataType: 'json'

            }).done(function(response) {
                if (response.error == 1) {
                    showMessage(_.isUndefined(i18n['No available services']) ? 'No available services': i18n['No available services']);
                    return false;
                } else {
                    var enabledServices = response.responseText.enabledServices,
                        dialog = _.template(LeadActionEmailServiceTemplate, {
                        enabledServices:enabledServices,
                        i18n:i18n
                    });

                    $(dialog).dialog({
                        width: '30%',
                        height: '300',
                        dialogClass: 'seotoaster',
                        resizable:false,
                        'title': _.isUndefined(i18n['Email Marketing'])?'Email Marketing':i18n['Email Marketing'],
                        open: function(event, ui) {
                            $('#marketing-services').on('change',  function() {
                                if ($("#marketing-services").val() != 'select') {
                                    $.ajax({
                                        url: $('#website_url').val()+'plugin/apps/run/getService/serviceName/'+$("#marketing-services").val(),
                                        type: 'GET',
                                        dataType: 'json'

                                    }).done(function(response) {
                                        if(response.error == 1){
                                            showMessage(_.isUndefined(i18n['No available lists'])?'No available lists':i18n['No available lists'], true, 5000);
                                            $('#subscribe-list').remove();
                                            $('#send-to-email-service').addClass('hidden');
                                            return false;
                                        }else{
                                            $('#subscribe-list').remove();
                                            var subscribeList = '<div class="grid_12 mt10px" id="subscribe-list">';
                                            $.each(response.responseText.list, function(value, listName){
                                                subscribeList += '<label class="fl-left mr30px pointer">'+listName+' <input type="checkbox" class="subscribe-to-email-service-list" name="list" value="'+value+'"/></label>'
                                            });
                                            subscribeList += '</div>';
                                            $('#marketing-services').after(subscribeList);
                                            $('#send-to-email-service').removeClass('hidden');
                                        }
                                    });
                                }else{
                                    $('#send-to-email-service').addClass('hidden');
                                    $('#subscribe-list').remove();
                                }
                            });

                            $('#sent-to-email-service-form').on('submit', function(e){
                                e.preventDefault();
                                var emailServiceName = $("#marketing-services").val(),
                                    checkedLists = $(this).find("input.subscribe-to-email-service-list:checked");

                                if(emailServiceName == 'select'){
                                    showMessage(_.isUndefined(i18n['Please choose service'])?'Please choose service':i18n['Please choose service'], true, 5000);
                                    return false;
                                }

                                if(checkedLists.length == 0){
                                    showMessage(_.isUndefined(i18n['Please choose list'])?'Please choose list':i18n['Please choose list'], true, 5000);
                                    return false;
                                }

                                var lists = [];
                                checkedLists.each(function() {
                                    lists.push($(this).val());
                                });

                                $.ajax({
                                    url: $(this).attr('action'),
                                    type: 'POST',
                                    dataType: 'json',
                                    'data' : {'secureToken': $('#lead-grid-token').val(), 'emailServiceName': emailServiceName, 'lists': lists.join(','), leadIds:checkedLeads}
                                }).done(function(response) {
                                    if(response.error == 0){
                                        showMessage(_.isUndefined(i18n['Emails added'])?'Emails added':i18n['Emails added'], false, 5000);
                                        $('.smoke').css({'z-index': 10000});
                                    }else{
                                        showMessage(_.isUndefined(i18n['Something went wrong'])?'Something went wrong':i18n['Something went wrong']);
                                    }
                                })
                            });

                        },
                        close: function(event, ui){
                            $(this).dialog('close').remove();
                        }
                    });
                    return false;
                }

            });
        },
        assignTotalOpportunitiesAmount: function(leadIds) {
            var self = this;
            $.get($('#website_url').val()+'api/leads/leadsgridstats/', {'leadIds' : leadIds.join(',')}, function(response){
                if (!_.isEmpty(response)) {
                    if (!_.isUndefined(response.totalSpentInfo)) {
                        _.each(response.totalSpentInfo, function (statData, id) {
                            self.$el.find('.lead-row-id-' + id).find('.total-sales-amount').text(accounting.formatMoney(Math.round(parseFloat(statData.total).toFixed(2)), JSON.parse(self.leads.moneyFormat)));
                        });
                    }

                    if (!_.isUndefined(response.totalOpportunitiesInfo)) {
                        _.each(response.totalOpportunitiesInfo, function (opportunityData, id) {
                            if (opportunityData.total !== null) {
                                self.$el.find('.lead-row-id-' + id).find('.total-opportunities-amount').html('<span class="total-opportunities-amount-value">'+accounting.formatMoney(Math.round(opportunityData.total), JSON.parse(self.leads.moneyFormat))+'</span>');
                            }
                        });
                    }
                }
            }, 'json');
        },
        callLeadDialog: function(e) {
            var leadId =  $(e.currentTarget).data('cid'),
                model = this.leads.get(leadId),
                mobile = '',
                mobileCountyCodeValue = '',
                mobileCountyCode = '',
                phone = '',
                phoneCountryCodeValue = '',
                phoneCountryCode = '',
                self = this,
                currentMobilePhone = '',
                currentDesktopPhone = '',
                currentMobileCountryCode = '',
                currentDesktopCountryCode = '';


                if (!_.isNull(model.get('lead_mobile'))) {
                    mobile = model.get('lead_mobile');
                }
                if (!_.isNull(model.get('lead_mobile_country_code_value'))) {
                    mobileCountyCodeValue = model.get('lead_mobile_country_code_value');
                }
                if (!_.isNull(model.get('lead_mobile_country_code'))) {
                    mobileCountyCode = model.get('lead_mobile_country_code');
                }

                if (!_.isNull(model.get('lead_phone'))) {
                    phone = model.get('lead_phone');
                }
                if (!_.isNull(model.get('lead_phone_country_code_value'))) {
                    phoneCountryCodeValue = model.get('lead_phone_country_code_value');
                }
                if (!_.isNull(model.get('lead_phone_country_code'))) {
                    phoneCountryCode = model.get('lead_phone_country_code');
                }
                if (!_.isNull(this.leads.currnetMobileCountryCode)) {
                    currentMobileCountryCode = this.leads.currnetMobileCountryCode;
                }
                if (!_.isNull(this.leads.currentMobilePhone)) {
                    currentMobilePhone = this.leads.currentMobilePhone;
                }
                if (!_.isNull(this.leads.currentDesktopCountryCode)) {
                    currentDesktopCountryCode = this.leads.currentDesktopCountryCode;
                }
                if (!_.isNull(this.leads.currentDesktopPhone)) {
                    currentDesktopPhone = this.leads.currentDesktopPhone;
                }

            var dialog = _.template(CallDialogTmpl, {
                i18n:i18n,
                mobile: mobile,
                phone: phone,
                mobileCountyCodeValue: mobileCountyCodeValue,
                phoneCountryCodeValue: phoneCountryCodeValue,
                mobileCountyCode: mobileCountyCode,
                phoneCountryCode: phoneCountryCode,
                phoneCountryCodes : self.leads.phoneCountryCodes,
                currentMobilePhone : currentMobilePhone,
                currentDesktopPhone: currentDesktopPhone,
                leadModel:model
            });

            $(dialog).dialog({
                dialogClass: 'seotoaster',
                buttons: {},
                resizable : false,
                height    : 150,
                width     : 1000,
                'title': 'Call to '+model.get('lead_first_name')+' '+model.get('lead_last_name'),
                open: function(event, ui) {
                    var desktopMasks = self.leads.desktopMasks,
                        mobileMasks = self.leads.mobileMasks;

                    if (typeof mobileMasks[mobileCountyCode] !== 'undefined') {
                        $('#call-lead-number').mask(mobileMasks[mobileCountyCode].full_mask_value, {autoclear: false});
                    } else {
                        $('#call-lead-number').mask('(999) 999 9999', {autoclear: false});
                    }

                    if (typeof mobileMasks[currentMobileCountryCode] !== 'undefined') {
                        $('#lead-from-call-mobile').mask(mobileMasks[currentMobileCountryCode].mask_value, {autoclear: false});
                    } else {
                        $('#lead-from-call-mobile').mask('(999) 999 9999', {autoclear: false});
                    }
                    var mobile = _.isUndefined(i18n['Mobile'])?'Mobile':i18n['Mobile'];
                    $('#lead-from-call-mobile-number').text(mobile + ' ' + $('#lead-from-call-mobile').val());

                    if (typeof mobileMasks[currentDesktopCountryCode] !== 'undefined') {
                        $('#lead-from-call-desktop').mask(mobileMasks[currentDesktopCountryCode].mask_value, {autoclear: false});
                    } else {
                        $('#lead-from-call-desktop').mask('(999) 999 9999', {autoclear: false});
                    }
                    var desktop = _.isUndefined(i18n['Desktop'])?'Desktop':i18n['Desktop'];
                    $('#lead-from-call-desktop-number').text(desktop + ' ' + $('#lead-from-call-desktop').val());

                    $('#call-lead-action').on('click',  function(){
                        $.ajax({
                            url: $('#website_url').val()+'plugin/leads/run/callLead/',
                            type: 'POST',
                            dataType: 'json',
                            data:{'secureToken':$('#lead-grid-token').val(), 'lead-call-country-code':$('#lead-phone-mobile-country-codes').val(), 'lead-call-number':$('#call-lead-number').val(), 'call-from-device-type':$('#call-from-device-type').val(), 'leadId':leadId, 'i18n' : i18n}
                        }).done(function(response){
                            showMessage(response.responseText, false, 5000);
                            $('#call-lead-dialog').dialog('close');
                        }).fail(function(response){
                            showMessage(response.responseJSON, true, 5000);
                        });
                    });
                    $('#call-device-type').on('change',  function(e){
                         var deviceTypeEl = $(e.currentTarget),
                            deviceType = deviceTypeEl.val(),
                            phoneEl = $('#call-lead-number');

                        if (deviceType === 'mobile') {
                            if (_.isEmpty(mobileCountyCode)) {
                                $('#lead-phone-mobile-country-codes').val('AC');
                            } else {
                                $('#lead-phone-mobile-country-codes').val(mobileCountyCode);
                            }
                            phoneEl.val(mobile);
                            if (typeof mobileMasks[mobileCountyCode] !== 'undefined') {
                                $('#call-lead-number').mask(mobileMasks[mobileCountyCode].full_mask_value, {autoclear: false});
                            } else {
                                $('#call-lead-number').mask('(999) 999 9999', {autoclear: false});
                            }
                        }

                        if (deviceType === 'desktop') {
                            if (_.isEmpty(phoneCountryCode)) {
                                $('#lead-phone-mobile-country-codes').val('AC');
                            } else {
                                $('#lead-phone-mobile-country-codes').val(phoneCountryCode);
                            }
                            phoneEl.val(phone);
                            if (typeof desktopMasks[phoneCountryCode] !== 'undefined') {
                                $('#call-lead-number').mask(desktopMasks[phoneCountryCode].full_mask_value, {autoclear: false});
                            } else {
                                $('#call-lead-number').mask('(999) 999 9999', {autoclear: false});
                            }
                        }
                    });

                    $('#lead-phone-mobile-country-codes').on('change',  function(e){
                       var countryEl = $(e.currentTarget),
                           chosenCountryCode = countryEl.val(),
                           deviceType = $('#call-device-type').val();

                        if (deviceType === 'mobile') {
                            if (typeof mobileMasks[chosenCountryCode] !== 'undefined') {
                                $('#call-lead-number').mask(mobileMasks[chosenCountryCode].full_mask_value, {autoclear: false});
                            } else {
                                $('#call-lead-number').mask('(999) 999 9999', {autoclear: false});
                            }
                        }

                        if (deviceType === 'desktop') {
                            if (typeof desktopMasks[chosenCountryCode] !== 'undefined') {
                                $('#call-lead-number').mask(desktopMasks[chosenCountryCode].full_mask_value, {autoclear: false});
                            } else {
                                $('#call-lead-number').mask('(999) 999 9999', {autoclear: false});
                            }
                        }
                    });
                },
                close: function(event, ui){
                    $(this).dialog('close').remove();
                }
            });
            return false;
        },
        editLeadAttribute: function(e)
        {
            var currentAttribute = $(e.currentTarget),
                currentValue = currentAttribute.val(),
                oldValue = currentAttribute.data('old-value'),
                leadId = currentAttribute.data('cid'),
                leadModel = this.leads.get(leadId),
                attrName =  currentAttribute.data('attrName'),
                self = this;

            if (currentValue != oldValue) {
                leadModel.set(attrName, currentValue);
                leadModel.set('secureToken', $('#lead-grid-token').val());
                leadModel.save(null, {
                    success: function(model, response){
                        if (response.error == '1') {
                            if (typeof response.responseText.duplicatedLeadEmail !== 'undefined') {
                                showConfirm(response.responseText.message + ' ' + (_.isUndefined(i18n['Would you like to merge both records?']) ? 'Would you like to merge both records?' : i18n['Would you like to merge both records?']), function () {
                                    var mergeToId = response.responseText.mergeToId,
                                        mergeBasedOn = response.responseText.mergeBasedOn;
                                    leadModel.set('mergeRecordWithId', mergeToId);
                                    leadModel.set('mergeBasedOn', mergeBasedOn);
                                    leadModel.save(null, {
                                        success: function (model, response) {
                                            window.location = $('#website_url').val() + 'dashboard/leads/#lead/' + mergeToId
                                        },
                                        error: function (model, response) {
                                            leadModel.set('mergeRecordWithId', '');
                                            leadModel.set('mergeBasedOn', '');
                                            leadModel.set(attrName, currentAttribute.data('lead-old-val'));
                                            showMessage(response.responseJSON, true, 5000);
                                            currentAttribute.val(currentAttribute.data('lead-old-val'));
                                            return false;
                                        }
                                    });
                                }, function () {
                                    leadModel.set(attrName, currentAttribute.data('lead-old-val'));
                                    currentAttribute.val(currentAttribute.data('lead-old-val'));
                                });
                            } else {
                                showMessage(response.responseText.message, true, 5000);
                            }
                        } else {
                            currentAttribute.data('old-value', currentValue);
                            showMessage(response.responseText.message, false, 5000);
                        }
                    },
                    error: function(model, response){
                        currentAttribute.val(oldValue);
                        showMessage(response.responseJSON, true, 5000);
                    }
                });

            }

        },
        deleteLead: function(e){
            var cid = $(e.currentTarget).data('cid'),
                model = this.leads.get(cid);

            showConfirm(_.isUndefined(i18n['Are you sure want to delete'])?'Are you sure want to delete':i18n['Are you sure want to delete'], function(){
                if (model){
                    model.destroy();
                }
            });
        },
        specialPageNavigator: function(pageNumber){
            var page = pageNumber;
            if ($.isNumeric(page)){
                this.leads.goTo(page);
            }
        },
        navigate: function(e){
            e.preventDefault();

            var page = $(e.currentTarget).data('page');
            if ($.isNumeric(page)){
                this.leads.goTo(page);
            } else {
                switch(page){
                    case 'first':
                        this.leads.goTo(this.leads.firstPage);
                        break;
                    case 'last':
                        this.leads.goTo(this.leads.information.lastPage);
                        break;
                    case 'prev':
                        this.leads.requestPreviousPage();
                        break;
                    case 'next':
                        this.leads.requestNextPage();
                        break;
                }
            }
        },
        displayEmailTab: function(e)
        {
            e.preventDefault();

            var leadId = $(e.currentTarget).data('cid'),
                params = {};
            if (!leadId) {
                return false;
            }

            params.openTabName = 'email';
            window.appLeadRouter.leadDetails(leadId, params);

        }
    });

    return LeadsGridView;
});