define([
    'backbone',
    '../models/lead-opportunity',
    '../models/lead-opportunity-quote',
    'text!../templates/lead-opportunity-detailed-tab.html',
    '../collections/lead-opportunities',
    'i18n!../../../nls/dashboard_'+$('#main-dashboard-system-language').val()+'_ln',
    'moment'
], function(Backbone, LeadOpportunityModel, LeadOpportunityQuoteModel, LeadOpportunityTmpl, LeadOpportunityCollection, i18n, moment){
    var LeadOpportunityFormView = Backbone.View.extend({
        el: $('#lead-detailed-view-config-tab-opportunity'),
        events: {
            'submit #lead-tab-opportunity-form': 'submit',
            'submit #lead-tab-opportunity-quote-form': 'createQuote',
            'click .lead-tab-opportunity-switch':'switchOpportunity',
            'change .lead-form-opportunity': 'chooseExistingOpportunity',
            'keypress .lead-form-opportunity-value' : 'checkInputOpportunityValue'
        },
        templates: {

        },
        initialize: function(leadModel, leadOpportunitiesData){
            this.leadModel = leadModel;
            this.leadOpportunitiesData = leadOpportunitiesData;
        },
        render: function(){
            this.$el.empty();
            this.$el.html(_.template(LeadOpportunityTmpl, {'leadModel': this.leadModel, 'leadOpportunitiesData': this.leadOpportunitiesData, 'i18n': i18n}));
            this.$el.attr('action', $('#website_url').val()+'api/leads/opportunity');
            $('#lead-form-opportunity-expected-close-date').datepicker({
                dateFormat: 'd-M-yy',
                defaultDate: "+1w",
                changeMonth: true,
                changeYear: true,
                yearRange: "c-5:c+5",
                minDate: 1
            });

            if (typeof this.leadOpportunitiesData.checkedOpportunityId !== 'undefined' && this.leadOpportunitiesData.checkedOpportunityId !== '') {
                var opportunityTabEl = $('#lead-detailed-view-config-tabs-tab').find('[data-tab-id="lead-detailed-view-config-tab-opportunity"]');
                opportunityTabEl.data('checked-opportunity-id', '');
                this.$el.find('#lead-tab-existing-opportunity').trigger('click');
                this.$el.find('.lead-form-opportunity').val(this.leadOpportunitiesData.checkedOpportunityId).trigger('change');
            }

            return this;
        },
        chooseExistingOpportunity: function(e)
        {
            var switchOpportunity = $(e.currentTarget),
                opportunityForm = switchOpportunity.closest('form'),
                opportunitySwitchVal = switchOpportunity.val(),
                opportunityId = opportunityForm.find('.lead-form-opportunity').val();

            if (opportunitySwitchVal == '0') {
                opportunityForm.find('.lead-form-opportunity-name').addClass('hidden');
                opportunityForm.find('.lead-form-opportunity').removeClass('grid_4').addClass('grid_7');
                opportunityForm.find('.lead-form-opportunity-name').removeClass('grid_3').addClass('grid_4');
                //opportunityForm.find('#lead-form-opportunity-expected-close-date').closest('p').removeClass('grid_2').addClass('grid_3');
                //opportunityForm.find('.lead-form-opportunity-value').closest('p').removeClass('grid_2').addClass('grid_3');
            } else {
                var opportunityModel = this.leadOpportunitiesData.get(opportunityId);
                opportunityForm.find('.lead-form-opportunity-name').val(opportunityModel.get('opportunity_name')).removeClass('hidden');
                opportunityForm.find('.lead-form-opportunity').removeClass('grid_7').addClass('grid_4');
                opportunityForm.find('.lead-form-opportunity-name').removeClass('grid_4').addClass('grid_3');
                opportunityForm.find('.lead-opportunity-type-selection').val(opportunityModel.get('opportunity_type_id'));
                opportunityForm.find('.lead-form-opportunity-value').val(parseInt(opportunityModel.get('opportunity_value'), 10));
                opportunityForm.find('.lead-form-opportunity-stages').val(opportunityModel.get('opportunity_stage_id'));
                if (!_.isEmpty(opportunityModel.get('expected_close_date')) && opportunityModel.get('expected_close_date') !== '0000-00-00 00:00:00') {
                    $('#lead-form-opportunity-expected-close-date').val(moment(opportunityModel.get('expected_close_date'), 'YYYY-MM-DD HH:mm:ss').format('DD-MMM-YYYY'));
                } else {
                    $('#lead-form-opportunity-expected-close-date').val('');
                }
                opportunityForm.find('#lead-form-opportunity-expected-close-date').closest('p').removeClass('grid_3').addClass('grid_2');
            }
        },
        switchOpportunity: function(e){
            var switchEl = $(e.currentTarget),
                opportunityForm = switchEl.closest('form'),
                opportunitySwitchVal = switchEl.val();

            if (opportunitySwitchVal === 'old') {
                opportunityForm.find('.lead-form-opportunity').val(0).removeClass('hidden');
                opportunityForm.find('.lead-form-opportunity-name').removeClass('grid_7').addClass('grid_4').addClass('hidden');
                opportunityForm.find('.lead-form-opportunity').removeClass('grid_4').addClass('grid_7');
                opportunityForm.find('.create-quote-and-opportunity').val('Update opportunity');
            } else {
                opportunityForm.find('.lead-form-opportunity-name').removeClass('grid_4').addClass('grid_7').val('').removeClass('hidden');
                opportunityForm.find('.lead-form-opportunity').removeClass('grid_7').addClass('grid_4');
                opportunityForm.find('.lead-form-opportunity-stages').val('1').removeClass('hidden');
                opportunityForm.find('.lead-form-opportunity').addClass('hidden');
                opportunityForm.find('.create-quote-and-opportunity').val('Create opportunity');
                //opportunityForm.find('.lead-form-opportunity-value').closest('p').removeClass('grid_2').addClass('grid_3');
                //opportunityForm.find('#lead-form-opportunity-expected-close-date').closest('p').removeClass('grid_2').addClass('grid_3');
            }
        },
        createQuote: function(e)
        {
            e.preventDefault();

             var self = this,
                form = $(e.currentTarget),
                isValid = true,
                leadId = form.data('lead-id'),
                opportunityId = form.find('.opportunity-id').val(),
                quoteLabel = form.find('.quote-label').val();

            _.each(form.find('.required'), function(el){
                if (!$(el).val()){
                    isValid = false;
                }
            });

            this.leadOpportunityQuoteModel = new LeadOpportunityQuoteModel();

            this.leadOpportunityQuoteModel.set('secureToken', $('#lead-grid-token').val());
            this.leadOpportunityQuoteModel.set('opportunity_id', opportunityId);
            this.leadOpportunityQuoteModel.set('lead_id', leadId);
            this.leadOpportunityQuoteModel.set('quote_title', quoteLabel);
            this.leadOpportunityQuoteModel.save(null, {
                success: function(model, response){
                    showMessage(response.responseText.message, false, 5000);
                    //window.open($('#website_url').val()+response.responseText.quoteLink , '_blank');
                    self.leadOpportunityDataModel = new LeadOpportunityCollection();
                    self.leadOpportunityDataModel.fetch({data: {'excludeEmptyOpportunityStages':true, 'lead_id': leadId},
                        'success': function(leadOpportunityDataModel){
                            self.leadOpportunitiesData = leadOpportunityDataModel;
                            self.render();
                            $('#lead-detailed-view-statistics-tab-opportunity').trigger('opportunitiesQuotes:created');

                        }
                    });
                },
                error: function(model, response){
                    showMessage(response.responseText, true, 5000);
                }
            });
            return false;

        },
        checkInputOpportunityValue: function (e) {
            var char = e.key;
            var validChar = parseInt(char);

           if(isNaN(validChar)){
               showMessage(_.isUndefined(i18n['You can use only whole numbers'])?'You can use only whole numbers':i18n['You can use only whole numbers'], true);
               return false;
           }
        },
        submit: function(e){
            e.preventDefault();

            var self = this,
                form = $(e.currentTarget),
                isValid = true,
                leadId = form.data('lead-id'),
                opportunityType = $('.lead-tab-opportunity-switch:checked').val(),
                stageId = form.find('.lead-form-opportunity-stages').val(),
                opportunityName = form.find('.lead-form-opportunity-name').val(),
                createQuote = form.find('.lead-form-opportunity-create-quote').is(':checked'),
                leadOpportunityValue = form.find('.lead-form-opportunity-value').val(),
                expectedCloseDate = ($('#lead-form-opportunity-expected-close-date').val()) ? $.datepicker.formatDate('yy-mm-dd', $('#lead-form-opportunity-expected-close-date').datepicker("getDate")): '',
                opportunityTypeId = $('#lead-opportunity-type-selection').val();

            _.each(form.find('.required'), function(el){
                if (!$(el).val()){
                    isValid = false;
                }
            });

            if (!isValid){
                showMessage(_.isUndefined(i18n['Missing required field'])?'Missing required field':i18n['Missing required field'], true);
                return false;
            }

            if (opportunityType === 'old') {
                var opportunityId = form.find('.lead-form-opportunity').val(),
                    opportunityModel = this.leadOpportunitiesData.get(opportunityId);
                if (!opportunityModel) {
                    showMessage(_.isUndefined(i18n['Wrong id'])?'Wrong id':i18n['Wrong id'], true, 5000);
                    return false;
                }
            } else {
                opportunityModel = new LeadOpportunityModel();
            }

            opportunityModel.set('secureToken', $('#lead-grid-token').val());
            opportunityModel.set('opportunity_stage_id', stageId);
            opportunityModel.set('opportunity_value', leadOpportunityValue);
            opportunityModel.set('expected_close_date', expectedCloseDate);
            opportunityModel.set('lead_id', leadId);
            opportunityModel.set('opportunity_name', opportunityName);
            opportunityModel.set('opportunity_type_id', opportunityTypeId);
            opportunityModel.save(null, {
                success: function(model, response){
                    showMessage(response.responseText.message, false, 5000);
                    if (createQuote) {
                        $('#lead-tab-opportunity-quote-form').removeClass('hidden');
                        $('#lead-tab-opportunity-form').addClass('hidden');
                        $('#lead-tab-opportunity-quote-form').find('.opportunity-id').val(response.responseText.opportunityId);
                    } else {
                        self.leadOpportunityDataModel = new LeadOpportunityCollection();
                        self.leadOpportunityDataModel.fetch({data: {'excludeEmptyOpportunityStages':true, 'lead_id': leadId},
                            'success': function(leadOpportunityDataModel){
                                self.leadOpportunitiesData = leadOpportunityDataModel;
                                self.render();

                            }
                        });
                    }
                },
                error: function(model, response){
                    showMessage(response.responseText, true, 5000);
                }
            });
        }
    });

    return LeadOpportunityFormView;
});