define([
    'backbone',
    'i18n!../../../nls/dashboard_'+$('#main-dashboard-system-language').val()+'_ln'
], function(Backbone, i18n){
    var LeadOrganizationFilterPresetView = Backbone.View.extend({
        el: $('#lead-organization-additional-filters'),
        events: {
            'click #save-filter-preset': 'saveFilterPreset',
            'change #lead-organization-predefined-filter-list': 'changeFilterPreset'
        },
        templates: {

        },
        initialize: function(){
            this.$el = $('#lead-organization-additional-filters');
        },
        render: function(){
            return this;
        },
        saveFilterPreset: function(e)
        {
            e.preventDefault();
            var leadOrganizationFilterPresetName = $('#lead-organization-filter-preset-name').val(),
                leadOrganizationFilterPresetDefault = 0,
                leadOrganizationFilterAllowPreset = 'individual',
                requestType = 'POST',
                leadPresetId = $('#lead-organization-preset-id').val(),
                filterPresetData = {};
            if (!_.isEmpty(leadOrganizationFilterPresetName)) {
                if ($('#lead-organization-filter-preset-default').is(':checked')) {
                    leadOrganizationFilterPresetDefault = 1;
                }
                if ($('#lead-organization-filter-preset-allow').is(':checked')) {
                    leadOrganizationFilterAllowPreset = 'all';
                }
                if (!_.isEmpty(leadPresetId)) {
                    requestType = 'PUT';
                }

                filterPresetData = {'created_at_from' : ($('#lead-organization-filter-from-date').val()) ? $.datepicker.formatDate('yy-mm-dd', $('#lead-organization-filter-from-date').datepicker("getDate")): '',
                    'created_at_to': ($('#lead-organization-filter-to-date').val()) ? $.datepicker.formatDate('yy-mm-dd', $('#lead-organization-filter-to-date').datepicker("getDate")): '',
                    'customer_group_id':$('#lead-organization-filter-by-group').val(),
                    'organization_zip': $('#lead-filter-organization-zip').val(),
                    'organization_country': $('#lead-organization-filter-country').val(),
                    'organization_city': $('#lead-filter-organization-city').val(),
                    'total_opportunity_amount_from': $('#lead-filter-total-opportunity-amount-from').val(),
                    'total_opportunity_amount_to': $('#lead-filter-total-opportunity-amount-to').val(),
                    'organization_by_industry':$('#lead-organization-filter-by-industry').val(),
                    'last_interaction_id':$('#lead-organization-filter-by-last-interaction').val(),
                    'industry_id':$('#lead-organization-filter-by-industry').val(),
                    'created_at': $('#switch-created-date:checked').val(),
                    'last_updated': $('#switch-updated-date:checked').val(),
                    'lead_opportunity_stages': $('#lead-filter-organization-opportunity-stage').val(),
                    'opportunity_type_id':$('#lead-filter-by-opportunity-type').val()
                };

                var formParams = {'filter_preset_type': $('#filter-preset-type').val(), 'filter_preset_name':leadOrganizationFilterPresetName,'is_default': leadOrganizationFilterPresetDefault,
                    'access': leadOrganizationFilterAllowPreset, 'filter_preset_data': filterPresetData, 'secureToken': $('#lead-organization-grid-token').val()
                    };
                if (requestType === 'PUT') {
                    formParams.id = leadPresetId;
                    formParams = JSON.stringify(formParams);
                }

                $.ajax({
                    'url': $('#website_url').val() + 'api/leads/leadfilterpreset/',
                    'type' : requestType,
                    'dataType': 'json',
                    'data': formParams
                }).done(function(response){
                    $('#lead-organization-preset-id').val('');
                    $('#lead-organization-filter-preset-default').prop('checked', false);
                    $('#lead-organization-filter-preset-allow').prop('checked', false);
                    $('#lead-organization-filter-preset-name').val('');
                    if (requestType === 'POST') {
                        $('#lead-organization-predefined-filter-list').append('<option value="' + response.responseText.id + '">' + leadOrganizationFilterPresetName + '</option>');
                    } else {
                        $('#lead-organization-predefined-filter-list option:selected').text(leadOrganizationFilterPresetName);
                    }
                    $('#lead-organization-predefined-filter-list').val(0);
                    showMessage(response.responseText.message, false, 5000);
                    $('#lead-organization-filter-reset').trigger('click');
                }).fail(function(response) {
                    showMessage(response.responseJSON, true, 5000);
                });
            }
        },
        changeFilterPreset:function(e)
        {
            var presetId = $(e.currentTarget).val();
            if (_.isEmpty(presetId)) {
                return false;
            }

            if (presetId == '0') {
                $('#lead-organization-preset-id').val('');
                $('#lead-organization-filter-reset').trigger('click');
                return false;
            }

            $.ajax({
                'url': $('#website_url').val()+'api/leads/leadfilterpreset/',
                'type':'GET',
                'dataType':'json',
                'data': {'id': presetId, 'secureToken': $('#lead-grid-token').val()}
            }).done(function(responseData){
                if (responseData.isDefault == '1') {
                    $('#lead-organization-filter-preset-default').prop('checked', true);
                } else {
                    $('#lead-organization-filter-preset-default').prop('checked', false);
                }
                if (responseData.access === 'individual') {
                    $('#lead-organization-filter-preset-allow').prop('checked', false);
                } else {
                    $('#lead-organization-filter-preset-allow').prop('checked', true);
                }
                var filtersData = JSON.parse(responseData.filterPresetData),
                    fromDate = '',
                    toDate = '';

                if (filtersData.created_at_from !== '') {
                    fromDate = $.datepicker.formatDate('d-M-yy', new Date(filtersData.created_at_from));
                }
                if (filtersData.created_at_to !== '') {
                    toDate = $.datepicker.formatDate('d-M-yy', new Date(filtersData.created_at_to));
                }

                $('#lead-organization-filter-from-date').val(fromDate);
                $('#lead-organization-filter-to-date').val(toDate);
                if (filtersData.customer_group_id) {
                    $('#lead-organization-filter-by-group').val(filtersData.customer_group_id);
                } else {
                    $('#lead-organization-filter-by-group').val(0);
                }

                if (filtersData.last_interaction_id) {
                    $('#lead-organization-filter-by-last-interaction').val(filtersData.last_interaction_id);
                } else {
                    $('#lead-organization-filter-by-last-interaction').val(0);
                }
                if (filtersData.industry_id) {
                    $('#lead-organization-filter-by-industry').val(filtersData.industry_id);
                } else {
                    $('#lead-organization-filter-by-industry').val(0);
                }
                $('#lead-filter-total-opportunity-amount-from').val(filtersData.total_opportunity_amount_from);
                $('#lead-filter-total-opportunity-amount-to').val(filtersData.total_opportunity_amount_to);
                $('#lead-filter-organization-city').val(filtersData.organization_city);
                $('#lead-filter-organization-zip').val(filtersData.organization_zip);
                if (filtersData.organization_country) {
                    $('#lead-organization-filter-country').val(filtersData.organization_country);
                } else {
                    $('#lead-organization-filter-country').val(0);
                }
                $('#lead-organization-filter-preset-name').val(responseData.filterPresetName);
                $('#lead-organization-preset-id').val(presetId);

                if (filtersData.opportunity_type_id) {
                    $('#lead-filter-by-opportunity-type').val(filtersData.opportunity_type_id);
                } else {
                    $('#lead-filter-by-opportunity-type').val(0);
                }
                if (!_.isUndefined(filtersData.last_updated) && filtersData.last_updated === 'last_updated') {
                    $('#switch-updated-date').prop('checked', true);
                } else {
                    $('#switch-updated-date').prop('checked', false);
                    $('#switch-created-date').prop('checked', true);
                }

                if (!_.isUndefined(filtersData.lead_opportunity_stages) && !_.isEmpty(filtersData.lead_opportunity_stages)) {
                    $('#lead-filter-organization-opportunity-stage').val(filtersData.lead_opportunity_stages).trigger("chosen:updated");
                } else {
                    $('#lead-filter-organization-opportunity-stage').val(0).trigger("chosen:updated");
                }

                $('#lead-organization-filter-apply').trigger('click');
            }).fail(function(response) {
                showMessage(response.responseJSON, true, 5000);
            });
        }
    });

    return LeadOrganizationFilterPresetView;
});