define([
    'backbone',
    '../collections/organizations',
    'text!../templates/paginator.html',
    'text!../templates/lead-organization-row.html',
    'i18n!../../../nls/dashboard_'+$('#main-dashboard-system-language').val()+'_ln',
    'moment',
    'chosen',
    'accounting'
], function(Backbone,
            LeadOrganizationsCollection, PaginatorTmpl, LeadOrganizationRowTmpl, i18n, moment, chosen, accounting
){

    var LeadsOrganizationsGridView = Backbone.View.extend({
        el: $('#lead-organization-grid-table'),
        events: {
            'click a[data-role=delete]': 'deleteOrganization',
            'click td.lead-organization-grid-paginator a.page': 'navigate',
            'blur input.lead-organization-edit-attribute': 'editOrganizationAttribute',
            'change select.lead-organization-edit-attribute': 'editOrganizationAttribute'
        },
        templates: {
            paginator: _.template(PaginatorTmpl)
        },
        initialize: function(options){
            this.$el.unbind();
            this.organizations = new LeadOrganizationsCollection();
            this.organizations.isGrid = true;

            this.organizations.on('reset', this.renderOrganizations, this);
            this.organizations.on('add', this.renderOrganizations, this);
            this.organizations.on('destroy', this.renderOrganizations, this);

            var filterFromDate = $('#lead-organization-filter-from-date'),
                filterToDate = $('#lead-organization-filter-to-date');

            filterFromDate.datepicker({
                dateFormat: 'd-M-yy',
                defaultDate: "+1w",
                changeMonth: true,
                changeYear: true,
                yearRange: "c-5:c+5",
                onSelect: function(selectedDate){
                    filterToDate.datepicker("option", "minDate", selectedDate);
                }
            });
            filterToDate.datepicker({
                dateFormat: 'd-M-yy',
                defaultDate: "+1w",
                changeMonth: true,
                changeYear: true,
                yearRange: "c-5:c+5",
                onSelect: function(selectedDate){
                    filterFromDate.datepicker("option", "maxDate", selectedDate);
                }
            });

            $('#lead-filter-organization-opportunity-stage').chosen();

        },
        render: function(){
            this.organizations.server_api = _.extend(this.organizations.server_api, {
                'filter': function() {
                    return {
                        'lead-organization-attribute': $('#lead-organization-attribute').val(),
                        'created_at_from' : ($('#lead-organization-filter-from-date').val()) ? $.datepicker.formatDate('yy-mm-dd', $('#lead-organization-filter-from-date').datepicker("getDate")): '',
                        'created_at_to': ($('#lead-organization-filter-to-date').val()) ? $.datepicker.formatDate('yy-mm-dd', $('#lead-organization-filter-to-date').datepicker("getDate")): '',
                        'last_interaction_id': $('#lead-organization-filter-by-last-interaction').val(),
                        'customer_group_id':$('#lead-organization-filter-by-group').val(),
                        'industry_id': $('#lead-organization-filter-by-industry').val(),
                        'organization_zip': $('#lead-filter-organization-zip').val(),
                        'organization_country': $('#lead-organization-filter-country').val(),
                        'organization_city': $('#lead-filter-organization-city').val(),
                        'total_opportunity_amount_from': $('#lead-filter-total-opportunity-amount-from').val(),
                        'total_opportunity_amount_to': $('#lead-filter-total-opportunity-amount-to').val(),
                        'last_updated': $('#switch-updated-date:checked').val(),
                        'created_at': $('#switch-created-date:checked').val(),
                        'lead_opportunity_stages': $('#lead-filter-organization-opportunity-stage').val(),
                        'opportunity_type_id':$('#lead-filter-by-opportunity-type').val()
                    };
                }
            });
            this.organizations.pager();
        },
        renderOrganizations: function(){
            this.$el.find('tbody').empty();
            this.organizations.each(this.renderOrganization, this);
            this.organizations.info()['i18n'] = i18n;
            this.$('td.lead-organization-grid-paginator').html(this.templates.paginator(this.organizations.information));
            this.assignTotalOpportunitiesAmount(this.organizations.pluck("id"));

            $(".lead-edit-organization").on("keydown", function(event) {
                if ( event.keyCode === $.ui.keyCode.TAB &&
                    $(this).autocomplete( "instance" ).menu.active) {
                    event.preventDefault();
                }
            }).autocomplete({
                    source: function(request, response) {
                        $.ajax({
                            'url': $('#website_url').val()+'api/leads/leadorganization/',
                            'type':'GET',
                            'dataType':'json',
                            'data': {searchTerm: request.term}
                        }).done(function(responseData){
                            if (!_.isEmpty(responseData)) {
                                response($.map(responseData, function (responseData) {
                                    return {
                                        label: responseData.organization_name,
                                        value: responseData.organization_name
                                    };
                                }));
                            }
                        });
                    },
                    search: function() {

                    },
                    focus: function() {
                        return false;
                    },
                    select: function(event, ui) {

                    }
            });
        },
        renderOrganization: function(leadsData){
            this.organizations.customerGroupsData = this.alphabeticalSort(this.organizations.customerGroups);
            this.organizations.organizationIndustriesData = this.alphabeticalSort(this.organizations.organizationIndustries);
            this.$el.find('tbody').append( _.template(LeadOrganizationRowTmpl, {'leadsOrganizationModel': leadsData,
                'customerGroups': this.organizations.customerGroupsData, 'organizationIndustries' : this.organizations.organizationIndustriesData, 'i18n': i18n}));
            var desktopMasks = this.organizations.desktopMasks,
                desktopCountryCode = leadsData.get('organization_phone_country_code');

            if (typeof desktopMasks[desktopCountryCode] !== 'undefined') {
                $('#lead-organization-edit-attribute-phone-'+leadsData.get('id')).mask(desktopMasks[desktopCountryCode].mask_value, {autoclear: false});
            } else {
                $('#lead-organization-edit-attribute-phone-'+leadsData.get('id')).mask('(999) 999 9999', {autoclear: false});
            }
        },
        assignTotalOpportunitiesAmount: function(organizationIds) {
            var self = this;
            $.get($('#website_url').val()+'api/leads/leadsorganizationgridstats/', {'organizationIds' : organizationIds.join(',')}, function(response){
                if (!_.isEmpty(response)) {
                    if (!_.isUndefined(response.totalSpentInfo)) {
                        _.each(response.totalSpentInfo, function (statData, id) {
                            self.$el.find('.organization-row-id-' + id).find('.total-sales-amount').text(accounting.formatMoney(Math.round(parseFloat(statData.total).toFixed(2)), JSON.parse(self.organizations.moneyFormat)));
                        });
                    }

                    if (!_.isUndefined(response.totalOpportunitiesInfo)) {
                        _.each(response.totalOpportunitiesInfo, function (opportunityData, id) {
                            if (opportunityData.total !== null) {
                                self.$el.find('.organization-row-id-' + id).find('.total-opportunities-amount').html('<span class="total-opportunities-amount-value">'+accounting.formatMoney(Math.round(opportunityData.total), JSON.parse(self.organizations.moneyFormat))+'</span>');
                            }
                        });
                    }

                    if (!_.isUndefined(response.interactionInfo)) {
                        _.each(response.interactionInfo, function (interactionData, id) {
                            self.$el.find('.organization-row-id-' + id).find('.last-interaction-type').text(interactionData.action_type_label);
                            self.$el.find('.organization-row-id-' + id).find('.last-interaction-date').text(moment(interactionData.created_at, 'YYYY-MM-DD HH:mm:ss').format('DD MMMM YYYY'));
                        });
                    }
                }
            }, 'json');
        },
        alphabeticalSort: function(obj){
            // convert object into array
            var sortable=[];
            for(var key in obj)
                if(obj.hasOwnProperty(key))
                    sortable.push([key, obj[key]]); // each item is an array in format [key, value]

            // sort items by value
            sortable.sort(function(a, b)
            {
                var x=a[1].toLowerCase(),
                    y=b[1].toLowerCase();
                return x<y ? -1 : x>y ? 1 : 0;
            });
            return sortable; // array in format [ [ key1, val1 ], [ key2, val2 ], ... ]
        },
        editOrganizationAttribute: function(e)
        {
            var currentAttribute = $(e.currentTarget),
                currentValue = currentAttribute.val(),
                oldValue = currentAttribute.data('old-value'),
                organizationId = currentAttribute.data('cid'),
                organizationModel = this.organizations.get(organizationId),
                attrName =  currentAttribute.data('attrName');

            if (currentValue != oldValue) {
                currentAttribute.data('old-value', currentValue);
                organizationModel.set(attrName, currentValue);
                organizationModel.set('secureToken', $('#lead-organization-grid-token').val());
                organizationModel.save(null, {
                    success: function(model, response){
                        showMessage(response.responseText.message, false, 5000);
                    },
                    error: function(model, response){
                        showMessage(response.responseJSON, true, 5000);
                    }
                });

            }

        },
        deleteOrganization: function(e){
            var cid = $(e.currentTarget).data('cid'),
                model = this.organizations.get(cid);

            showConfirm(_.isUndefined(i18n['Are you sure want to delete'])?'Are you sure want to delete':i18n['Are you sure want to delete'], function(){
                if (model){
                    model.destroy();
                }
            });
        },
        specialPageNavigator: function(pageNumber){
            var page = pageNumber;
            if ($.isNumeric(page)){
                this.organizations.goTo(page);
            }
        },
        navigate: function(e){
            e.preventDefault();

            var page = $(e.currentTarget).data('page');
            if ($.isNumeric(page)){
                this.organizations.goTo(page);
            } else {
                switch(page){
                    case 'first':
                        this.organizations.goTo(this.organizations.firstPage);
                        break;
                    case 'last':
                        this.organizations.goTo(this.organizations.information.lastPage);
                        break;
                    case 'prev':
                        this.organizations.requestPreviousPage();
                        break;
                    case 'next':
                        this.organizations.requestNextPage();
                        break;
                }
            }
        }
    });

    return LeadsOrganizationsGridView;
});